/*
Portions of the following code are modifications based on work created and shared by Google
in Earth Engine Data Catalog and Earth Engine Guides under the Apache 2.0 License. https://www.apache.org/licenses/LICENSE-2.0
*/

// NOTE: to run the script it is necessary to copy it into the Google Earth Engine (GEE) Code Editor,
//       therefore it is necessary being able to access to a Google account and to GEE;
//       it is also necessary to upload the shapefile "samplePlotsShapefile" (extensions .cpg, .dbf, .prj, .shp, .shx) into the GEE Assets
//       and, then, import it into the Code Editor and name it "plots" (without quotation marks)

// function to mask clouds, clouds shadows and snow
 
var maskCloudsShadowsSnow = function(image) {
  var bitCloudsShadow = (1 << 3); // clouds shadow
  var bitSnow = (1 << 4); // snow
  var bitClouds = (1 << 5);  // clouds
 
  var pixel_qa = image.select('pixel_qa');

  var mask = pixel_qa.bitwiseAnd(bitCloudsShadow).eq(0)
                     .and(pixel_qa.bitwiseAnd(bitSnow).eq(0))
                     .and(pixel_qa.bitwiseAnd(bitClouds).eq(0));
                 
  return image.updateMask(mask);
};

// function to add ndvi band to landsat 8 image

var addNDVI = function(image) {
  var ndvi = image.normalizedDifference(['B5', 'B4']).rename('NDVI');
  return image.addBands(ndvi);
};

// add ndvi band to landsat 8 image collection and mask clouds, clouds shadows and snow

var landsatCollectionWithNdvi = ee.ImageCollection('LANDSAT/LC08/C01/T1_SR')
                  .map(addNDVI)
                  .map(maskCloudsShadowsSnow);

// create list of image collections containing ndvi image of each month inside years range

var months = ee.List.sequence(1,12);

var imagesByMonth = months.map(function(m){
  m = ee.Number(m);
  
  var startDate = ee.Date.fromYMD(2015,m,1);
  var endDate = startDate.advance(1,'month');
  
  var monthMeanNdvi = landsatCollectionWithNdvi.select('NDVI')
                                          .filterDate(startDate, endDate)
                                          .mean();
  return monthMeanNdvi;
}).flatten(); // transform the collection in a list


/** function to add features to "plots" (point shapefile representing the locations of the sample plots) 
 * and to export the final product to Google Drive as a "CSV" file (one file for each month). 
 * 
 * The function adds fields regarding the following pieces of information:
 * 
 * standard deviation of NDVI values of each unmasked pixel inside the 500mx500m square buffers around each sample plot
 * mean of NDVI values of each pixel inside the buffers
 * count of unmasked pixels inside the buffers
 * total count of buffers' pixels (masked and unmasked)
 */
 
var final = function(n) {  // "n" represents the index of the image in the list

  var image = imagesByMonth.get(n); // get one object from the list...
  image = ee.Image(image);          // ...and transform it in a Google Earth Engine image

  // compute and add features to "plots"
  
  var finalTable = plots.map(function(feature) {
    var buffer500 = feature.buffer(250).bounds(); // create a buffer with radius = 250m (diameter = 500m) and make it square
    return feature.set('ndviSD',
                        image.reduceRegion({geometry: buffer500.geometry(),
                                            reducer: ee.Reducer.stdDev(),
                                            scale: 30,
                                            }) // the output of reduceRegion is a dictonary with "NDVI" as key and the std dev as value
                                            .get('NDVI')) // add only the std dev value to the table 
                  .set('ndviMean',
                        image.reduceRegion({geometry: buffer500.geometry(),
                                            reducer: ee.Reducer.mean(),
                                            scale: 30,
                                            }).get('NDVI'))
                  .set('unmaskedPixels',
                        image.reduceRegion({geometry: buffer500.geometry(),
                                            reducer: ee.Reducer.count(),
                                            scale: 30,
                                            }).get('NDVI'))
                  .set('totPixels',
                        image.unmask()
                             .reduceRegion({geometry: buffer500.geometry(),
                                            reducer: ee.Reducer.count(),
                                            scale: 30,
                                            }).get('NDVI'));
  });
                                                  
  return Export.table.toDrive({collection:finalTable,
                                description: 'month_' + (n+1).toString() + '_year_2015',
                                fileFormat: 'CSV'
                                });
  };


// create list of integers corresponding to the 12 months

var imageIndicesArray = [];
for (var i = 0; i < 12; i++) {imageIndicesArray.push(i)}

// make the final computation

imageIndicesArray.map(final);

// NOTE: it may take several minutes to export all 12 files

